unit uniGameClass;

interface

uses
  System.Classes, System.SysUtils,
  FMX.Objects, FMX.Controls, FMX.TabControl, FMX.Layouts, FMX.Types, System.Types;

type
  TGameScene =
    (
      NONE_SCENE,
      INTRO_SCENE,
      MAIN_SCENE,
      BEFORE_SCENE,
      RUN_SCENE,
      AFTER_SCENE,
      EXIT_SCENE,
      STOP_SCENE,
      PROGRAM_EXIT
    );

const
  NONE_READY          = $0000;
  STAND_BY            = $0001;

  GAMESTART_PRESS     = $0010;
  GAMEEND_PRESS       = $0020;

  CUPROLL_PRESS       = $0030;
  GAMESTOP_PRESS      = $0040;

  GAMECONTINUE_PRESS  = $0050;

  EXITOK_PRESS         = $0060;
  EXITCANCEL_PRESS     = $0070;

  STOPOK_PRESS        = $0080;
  STOPCANCEL_PRESS    = $0090;

  CUP_ROLL            = $0100;

  CUPNUM1_PRESS       = $0110;
  CUPNUM2_PRESS       = $0120;
  CUPNUM3_PRESS       = $0130;
  CUPNUM4_PRESS       = $0140;







  BEFORE_SCENE_TOP_MSG  = 'It is the current position of the ball.';
  AFTER_SCENE_TOP_MSG   = 'Please press the cup number that are hidden ball!';
  CUPROLL_MSG           = 'Roll!!! Roll!!!';
  BALLCHECK_FAIL_MSG    = 'Oh~~~. You failed!!!!';
  BALLCHECK_OK_MSG      = 'Wow~~~. Succeeded!!!!';

type

  TFadeParameter = record
    StartTime: Cardinal;
    EndTime: Cardinal;
    AllTime: Cardinal;
    Opacity: Single;
  end;

  TGameClass = class
  public
    constructor Create();
    destructor Destroy; override;
  public
    nowGameScene: TGameScene;
    oldGameScene: TGameScene;

    // Game Loop Check
    bGameLoop: Boolean;

    // Original Cup Image
    oriCupList: array[0..3] of TImage;
    // Main Page Control
    tbcMain: TTabControl;
    // Intor Page
    tbtmIntro: TTabItem;
    // Main Page
    tbtmMain: TTabItem;
    // Game Page
    tbtmBefore: TTabItem;
    // Exit Page
    tbtmExit: TTabItem;
    // Stop Page
    tbtmStop: TTabItem;

    // Main Page Scale Layout
    lytMainSceneMain: TScaledLayout;
    // rctnglMain: TRectangle;
    // Game Page Scale Layout
    lytBeforeSceneMain: TScaledLayout;
    // rctnglBefore: TRectangle;
    // Exit Page Scale Layout
    lytExitSceneMain: TScaledLayout;
    // rctnglExit: TRectangle;
    // Stop Page Scale Layout
    lytStopSceneMain: TScaledLayout;
    // rctnglStop: TRectangle;
  public

    // procedure SetCupList(CupList: array of TImage);

    // GameScene Init
    procedure InitGameScene;

    procedure SetGameScene(GameScene: TGameScene);
    procedure SetGameSceneIndexToStandBy(GameScene: TGameScene);
    function GetNowGameScene: TGameScene;
    function GetOldGameScene: TGameScene;

    // Scene Process Index Inint
    procedure InitSceneIndex(var Index: Integer);

    // FadeIn Init
    procedure InitFadeIn(var Control: Pointer; var FadeInTime: TFadeParameter);
    // FadeIn Process
    // Result = 0: Running , 1: Finish
    function RunFadeIn(var Control: Pointer; var FadeInTime: TFadeParameter): Integer;

    // FadeOut init
    procedure InitFadeOut(var Control: Pointer; var FadeOutTime: TFadeParameter);
    // FadeOut Process
    // Result = 0: Running , 1: Finish
    function RunFadeOut(var Control: Pointer; var FadeOutTime: TFadeParameter): Integer;

    // Result = 0: Running , 1: Finish
    function WorkProcess: Integer;
  public
    //==============================================================================
    // INTRO SCENE
    //==============================================================================
    intIntroSceneIndex: Integer;

    IntroSceneFadeInTime: TFadeParameter;
    IntroSceneFadeOutTime: TFadeParameter;
    txtIntro: TText;

    // Intro Scene Change
    procedure ChangeIntroScene;

    // Intor Index Init
    procedure InitIntroSceneIndex;
    // Intro FadeIn Init
    procedure InitIntroSceneFadeIn;
    // IntroScene FadeIn
    // Result = 0: Running , 1: Finish
    function IntroSceneFadeIn: Integer;

    // Intro FadeOut Init
    procedure InitIntroSceneFadeOut;
    // IntroScene FadeOut
    // Result = 0: Running , 1: Finish
    function IntroSceneFadeOut: Integer;

    // function IntroSceneFadeInOut: Integer; virtual; abstract;

    //
    procedure AllSceneScreenSetScale;
  public
    //==============================================================================
    // MAIN SCENE
    //==============================================================================
    intMainSceneIndex: Integer;
    MainSceneFadeInTime: TFadeParameter;
    MainSceneFadeOutTime: TFadeParameter;

    btnGameStart: TText;
    btnGameEnd: TText;

    imgBall: TImage;

    // Main Scene Change
    procedure ChangeMainScene;

    // Main Scene Index Init
    procedure InitMainSceneIndex;

    // Main FadeIn Init
    procedure InitMainSceneFadeIn;
    // Main FadeIn
    // Result = 0: Running , 1: Finish
    function MainSceneFadeIn: Integer;

    // ȭ FadeOut ʱȭ
    procedure InitMainSceneFadeOut;
    // ȭ FadeIn
    // Result = 0:  , 1: 
    function MainSceneFadeOut: Integer;

    //  GameStart  
    procedure GameStartPress;

    //  GameEnd  
    procedure GameEndPress;

    //   ʴ´
    // function MainSceneFadeInOut: Integer; virtual; abstract;

  public
    //==============================================================================
    // BEFORE SCENE
    //==============================================================================
    intBeforeSceneIndex: Integer;
    BeforeSceneFadeInTime: TFadeParameter;
    BeforeSceneFadeOutTime: TFadeParameter;

    // ӷ, ʰ 1
    intLevel: Integer;
    // ӿ ̹
    GameCupList: array[0..3] of TImage;
    // ġ
    intBallPosition: Integer;

    // ӷ ǥÿ
    txtGameLevel: TText;

    // ȭ  ޼
    txtGameTopMessage: TText;

    // ŵ 
    btnCupRoll: TText;
    //  
    btnGameStop: TText;

    btnCup1: TText;
    btnCup2: TText;
    btnCup3: TText;
    btnCup4: TText;

    // Before Scene ȭ ȯ
    procedure ChangeBeforeScene;

    // Before Scene Index ʱȭ
    procedure InitBeforeSceneIndex;

    // ӷ ʱȭ
    procedure InitLevel;
    // ӷ 
    procedure IncLevel;
    // ȭ鿡  ǥ
    procedure LevelDisplay;

    // ġ 
    procedure SetBallPosition;
    // ġ ǥ ( ܿ ǥ)
    procedure BallDisplay;
    procedure BallOnOff(Visible: Boolean);

    // ӻ ޼ 
    procedure SetGameTopMessage(Msg: String);
    procedure CupRollButtonOnOff(Visible: Boolean);
    procedure GameStopButtonOnOff(Visible: Boolean);
    procedure CupButtonEnableOnOff(Check: Boolean);

    //    ӿ ʱȭ
    procedure InitCup;
    //  ġ ǥ ( ǥ)
    procedure InitCupDisplay;

    // Before Scene FadeIn ʱȭ
    procedure InitBeforeSceneFadeIn;
    // Before Scene FadeIn
    // Result = 0:  , 1: 
    function BeforeSceneFadeIn: Integer;

    // Before Scene FadeOut ʱȭ
    procedure InitBeforeSceneFadeOut;
    // Before Scene FadeOut
    // Result = 0:  , 1: 
    function BeforeSceneFadeOut: Integer;

    //   
    procedure CupRollPress;
    //   
    procedure GameStopPress;

  public
    //==============================================================================
    // RUN SCENE
    //==============================================================================
    intRunSceneIndex: Integer;

    //  ȸ īƮ
    intMainCupRollCount: Integer;
    //  ȯ ε
    IntSubCupRollIndex: Integer;
    //  ȯ īƮ
    intSubCupRollCount: Integer;
    // ӷ  ȸ 
    intCupRollLevel: Single;
    // ȸ 
    intCupRollDelay: Cardinal;




    // Run Scene Index ʱȭ
    procedure InitRunSceneIndex;
    //  ȸ  ʱȭ
    procedure InitCupRollParameter;
    // ŵ  
    procedure InitCupRollDisplay;

    // ŵ
    // Result := 0  1: 
    function RunCupRollAction(Index: Integer): Integer;
  public
    //==============================================================================
    // AFTER SCENE
    //==============================================================================
    intAfterSceneIndex: Integer;
    btnGameContinue: TText;

    // After Scene Index ʱȭ
    procedure InitAfterSceneIndex;
    // ġ  Ȯ
    function GetBallPosition: Integer;
    // ġ ˻
    // Result := 0 Faile 1: OK
    function CheckBallPositionResult(Index: Integer): Integer;

    // ġ ǥ(ŵ  ǥ)
    procedure ResultBallDisplay;
    procedure ResultCupDisplay;

    procedure GameContinueOnOff(Visible: Boolean);

    // CupNumPress;
    procedure CupNum1Press;
    procedure CupNum2Press;
    procedure CupNum3Press;
    procedure CupNum4Press;

    // Game Continue Press
    procedure GameContinuePress;
  public
    //==============================================================================
    // EXIT SCENE
    //==============================================================================
    intExitSceneIndex: Integer;
    ExitSceneFadeInTime: TFadeParameter;
    ExitSceneFadeOutTime: TFadeParameter;

    btnExitOk: TText;
    btnExitCancel: TText;

    // Exit Scene ȯ
    procedure ChangeExitScene;

    // Exit Scene Index ʱȭ
    procedure InitExitSceneIndex;

    // Exit Scene FadeIn ʱȭ
    procedure InitExitSceneFadeIn;
    // Exit Scene FadeIn
    // Result = 0:  , 1: 
    function ExitSceneFadeIn: Integer;

    // Exit Scene FadeOut ʱȭ
    procedure InitExitSceneFadeOut;
    // Exit Scene FadeOut
    // Result = 0:  , 1: 
    function ExitSceneFadeOut: Integer;

    //   ʴ´
    // function ExitSceneFadeInOut: Integer; virtual; abstract;
    // function CheckGameExit(Index: Integer): Integer; virtual; abstract;

    // Exit Ok Press
    procedure ExitOkPress;
    // Exit Cancel Press;
    procedure ExitCancelPress;


  public
    //==============================================================================
    // STOP SCENE
    //==============================================================================
    intStopSceneIndex: Integer;
    StopSceneFadeInTime: TFadeParameter;
    StopSceneFadeOutTime: TFadeParameter;

    btnStopOk: TText;
    btnStopCancel: TText;

    // Stop Scene ȯ
    procedure ChangeStopScene;

    // Stop Scene Index ʱȭ
    procedure InitStopSceneIndex;

    // Stop Scene FadeIn ʱȭ
    procedure InitStopSceneFadeIn;
    // Stop Scene FadeIn
    // Result = 0:  , 1: 
    function StopSceneFadeIn: Integer;

    // Stop Scene FadeOut ʱȭ
    procedure InitStopSceneFadeOut;
    // Stop Scene FadeOut
    // Result = 0:  , 1: 
    function StopSceneFadeOut: Integer;

    procedure StopOkPress;
    procedure StopCancelPress;
  end;

implementation

{ TGameClass }

constructor TGameClass.Create;
begin

end;

destructor TGameClass.Destroy;
begin

  inherited;
end;

{
procedure TGameClass.SetCupList(CupList: array of TImage);
begin
  oriCupList[0].Tag := 0;
  oriCupList[1].Tag := 1;
  oriCupList[2].Tag := 2;
  oriCupList[3].Tag := 3;

  oriCupList[0] := CupList[0];
  oriCupList[1] := CupList[1];
  oriCupList[2] := CupList[2];
  oriCupList[3] := CupList[3];
end;
}
procedure TGameClass.SetGameScene(GameScene: TGameScene);
begin
  oldGameScene := nowGameScene;
  nowGameScene := GameScene;
end;

procedure TGameClass.SetGameSceneIndexToStandBy(GameScene: TGameScene);
begin
  case GameScene of
    NONE_SCENE: ;
    INTRO_SCENE: ;
    MAIN_SCENE: intMainSceneIndex := NONE_READY;
    BEFORE_SCENE: intBeforeSceneIndex := STAND_BY;
    RUN_SCENE: intRunSceneIndex := CUP_ROLL;
    AFTER_SCENE: intAfterSceneIndex := STAND_BY;
    EXIT_SCENE: ;
    STOP_SCENE: ;
    PROGRAM_EXIT: ;
  end;
end;

procedure TGameClass.InitGameScene;
begin
  nowGameScene := TGameScene.NONE_SCENE;
  oldGameScene := TGameScene.NONE_SCENE;
end;

function TGameClass.GetNowGameScene: TGameScene;
begin
  Result := nowGameScene;
end;

function TGameClass.GetOldGameScene: TGameScene;
begin
  Result := oldGameScene;
end;

procedure TGameClass.InitSceneIndex(var Index: Integer);
begin
  Index := NONE_READY;
end;

procedure TGameClass.InitFadeIn(var Control: Pointer; var FadeInTime: TFadeParameter);
begin
  if Not Assigned(Control) then Exit;

  // FadeIn ʱȭ
  TControl(Control).Opacity := 0;
  FadeInTime.StartTime := TThread.GetTickCount;
  FadeInTime.EndTime := TThread.GetTickCount + 1000 * 1;
  FadeInTime.AllTime := FadeInTime.EndTime - FadeInTime.StartTime;
  FadeInTime.Opacity := 0;
end;

function TGameClass.RunFadeIn(var Control: Pointer; var FadeInTime: TFadeParameter): Integer;
var
  intRunTime: Cardinal;
begin
  if Not Assigned(Control) then Exit;
  // FadeIn ó
  // Result = 0:  , 1: 
  if TThread.GetTickCount > FadeInTime.EndTime then
  begin
    // 
    FadeInTime.Opacity := 1;
    TControl(Control).Opacity := FadeInTime.Opacity;
    Result := 1;
  end
  else
  begin
    // 
    intRunTime := FadeInTime.EndTime - TThread.GetTickCount;
    FadeInTime.Opacity := intRunTime / FadeInTime.AllTime;
    TControl(Control).Opacity := 1 - FadeInTime.Opacity;
    Result := 0;
  end;
end;

procedure TGameClass.InitFadeOut(var Control: Pointer; var FadeOutTime: TFadeParameter);
begin
  if Not Assigned(Control) then Exit;

  // FadeOut ʱȭ
  TControl(Control).Opacity := 1;
  FadeOutTime.StartTime := TThread.GetTickCount;
  FadeOutTime.EndTime := TThread.GetTickCount + 1000 * 1;
  FadeOutTime.AllTime := FadeOutTime.EndTime - FadeOutTime.StartTime;
  FadeOutTime.Opacity := 1;
end;

function TGameClass.RunFadeOut(var Control: Pointer; var FadeOutTime: TFadeParameter): Integer;
var
  intRunTime: Cardinal;
begin
  if Not Assigned(Control) then Exit;
  // FadeOut ó
  // Result = 0:  , 1: 
  if TThread.GetTickCount > FadeOutTime.EndTime then
  begin
    // 
    FadeOutTime.Opacity := 0;
    TControl(Control).Opacity := FadeOutTime.Opacity;
    Result := 1;
  end
  else
  begin
    // 
    intRunTime := FadeOutTime.EndTime - TThread.GetTickCount;
    FadeOutTime.Opacity := intRunTime / FadeOutTime.AllTime;
    TControl(Control).Opacity := FadeOutTime.Opacity;
    Result := 0;
  end;
end;

//==============================================================================
// INTRO SCENE
//==============================================================================
procedure TGameClass.ChangeIntroScene;
begin
  // Ʈ ȭ ȯ
  tbcMain.ActiveTab := tbtmIntro;
  tbtmIntro.Opacity := 1;
end;

procedure TGameClass.InitIntroSceneIndex;
begin
  InitSceneIndex(intIntroSceneIndex);
end;

procedure TGameClass.InitIntroSceneFadeIn;
begin
  // Ʈ  FadeIn ʱȭ
  InitFadeIn(Pointer(txtIntro), IntroSceneFadeInTime);
end;

function TGameClass.IntroSceneFadeIn: Integer;
begin
  // IntroScene FadeIn
  // Result = 0:  , 1: 
  Result := RunFadeIn(Pointer(txtIntro), IntroSceneFadeInTime);
end;

procedure TGameClass.InitIntroSceneFadeOut;
begin
  // Ʈ  FadeOut ʱȭ
  InitFadeOut(Pointer(txtIntro), IntroSceneFadeOutTime);
end;

function TGameClass.IntroSceneFadeOut: Integer;
begin
  // IntroScene FadeOut
  // Result = 0:  , 1: 
  Result := RunFadeOut(Pointer(txtIntro), IntroSceneFadeOutTime);
end;

procedure TGameClass.AllSceneScreenSetScale;
var
  fScale: TPosition;
begin
  //  ȭ鿡 ߾   Ѵ.
  try
    fScale := TPosition.Create(TPoint.Zero);
    fScale.X := tbcMain.Width / 256;
    fScale.Y := tbcMain.Height / 192;
    //    ̾ƿ
    lytMainSceneMain.Scale := fScale;
    //    ̾ƿ
    lytBeforeSceneMain.Scale := fScale;
    // ȭ   ̾ƿ
    lytExitSceneMain.Scale := fScale;
    // ȭ   ̾ƿ
    lytStopSceneMain.Scale := fScale;

  finally
    // fScale.Destroy;
  end;
end;

//==============================================================================
// MAIN SCENE
//==============================================================================
procedure TGameClass.ChangeMainScene;
begin
  tbcMain.ActiveTab := tbtmMain;
  lytMainSceneMain.Opacity := 1;
end;

procedure TGameClass.InitMainSceneIndex;
begin
  // ȭ ε ʱȭ
  InitSceneIndex(intMainSceneIndex);
end;

procedure TGameClass.InitMainSceneFadeIn;
begin
  // ȭ FadeIn ʱȭ
  InitFadeIn(Pointer(lytMainSceneMain), MainSceneFadeInTime);
end;

function TGameClass.MainSceneFadeIn: Integer;
begin
  // ȭ FadeIn
  // Result = 0:  , 1: 
  Result := RunFadeIn(Pointer(lytMainSceneMain), MainSceneFadeInTime);
end;

procedure TGameClass.InitMainSceneFadeOut;
begin
  // ȭ FadeOut ʱȭ
  InitFadeOut(Pointer(lytMainSceneMain), MainSceneFadeOutTime);
end;

function TGameClass.MainSceneFadeOut: Integer;
begin
  // ȭ FadeOut
  // Result = 0:  , 1: 
  Result := RunFadeOut(Pointer(lytMainSceneMain), MainSceneFadeOutTime);
end;

procedure TGameClass.GameStartPress;
begin
  case intMainSceneIndex of
    GAMESTART_PRESS, GAMESTART_PRESS + 1:;
    GAMEEND_PRESS, GAMEEND_PRESS + 1:;
  else
    intMainSceneIndex := GAMESTART_PRESS;
  end;
end;

procedure TGameClass.GameEndPress;
begin
  case intMainSceneIndex of
    GAMESTART_PRESS, GAMESTART_PRESS + 1:;
    GAMEEND_PRESS, GAMEEND_PRESS + 1:;
  else
    intMainSceneIndex := GAMEEND_PRESS;
  end;
end;

//==============================================================================
// BEFORE SCENE
//==============================================================================
procedure TGameClass.ChangeBeforeScene;
begin
  // Before Scene ȭ ȯ
  tbcMain.ActiveTab := tbtmBefore;
  lytBeforeSceneMain.Opacity := 1;
end;
procedure TGameClass.InitBeforeSceneIndex;
begin
  // Before Scene Index ʱȭ
  InitSceneIndex(intBeforeSceneIndex);
end;

procedure TGameClass.InitLevel;
begin
  // ӷ ʱȭ
  intLevel := 1;
end;

procedure TGameClass.IncLevel;
begin
  Inc(intLevel);
end;

procedure TGameClass.LevelDisplay;
begin
  // ȭ鿡  ǥ
  txtGameLevel.Text := 'LEVEL ' + IntToStr(intLevel);
end;

procedure TGameClass.SetBallPosition;
begin
  // ġ  (0 ~ 3)
  intBallPosition := Random(4);
end;

procedure TGameClass.BallDisplay;
var
  intX, intY: Integer;
begin
  // ġ ǥ ( ܿ ǥ)
  intY := 84;
  case intBallPosition of
    0: intX := 72;
    1: intX := 104;
    2: intX := 136;
    3: intX := 168;
  end;

  imgBall.Position.X := intX;
  imgBall.Position.Y := intY;

  BallOnOff(True);
end;

procedure TGameClass.BallOnOff(Visible: Boolean);
begin
  imgBall.Visible := Visible;
end;

procedure TGameClass.SetGameTopMessage(Msg: String);
begin
  // ӻ ޼ 
  txtGameTopMessage.Text := Msg;
end;

procedure TGameClass.CupRollButtonOnOff(Visible: Boolean);
begin
  btnCupRoll.Visible := Visible;
end;

procedure TGameClass.GameStopButtonOnOff(Visible: Boolean);
begin
  btnGameStop.Visible := Visible;
end;

procedure TGameClass.CupButtonEnableOnOff(Check: Boolean);
begin
  btnCup1.Enabled := Check;
  btnCup2.Enabled := Check;
  btnCup3.Enabled := Check;
  btnCup4.Enabled := Check;
end;

procedure TGameClass.InitCup;
begin
  //    ӿ ʱȭ
  GameCupList[0] := oriCupList[0];
  GameCupList[1] := oriCupList[1];
  GameCupList[2] := oriCupList[2];
  GameCupList[3] := oriCupList[3];
end;

procedure TGameClass.InitCupDisplay;
var
  intLoop: Integer;
begin
  // Ball Visible False
  // BallOnOff(False);

  // Cup Visible Off
  for intLoop := 0 to 3 do
  begin
    GameCupList[intLoop].Visible := False;
  end;

  GameCupList[0].Position.X := 67;
  GameCupList[0].Position.Y := 78;
  GameCupList[1].Position.X := 99;
  GameCupList[1].Position.Y := 78;
  GameCupList[2].Position.X := 131;
  GameCupList[2].Position.Y := 78;
  GameCupList[3].Position.X := 163;
  GameCupList[3].Position.Y := 78;

  GameCupList[intBallPosition].Position.Y := 58;

  // Cup Visible On
  for intLoop := 0 to 3 do
  begin
    GameCupList[intLoop].Visible := True;
  end;
end;

procedure TGameClass.InitBeforeSceneFadeIn;
begin
  // Before Scene FadeIn ʱȭ
  InitFadeIn(Pointer(lytBeforeSceneMain), BeforeSceneFadeInTime);
end;

function TGameClass.BeforeSceneFadeIn: Integer;
begin
  // Before Scene FadeIn
  // Result = 0:  , 1: 
  Result := RunFadeIn(Pointer(lytBeforeSceneMain), BeforeSceneFadeInTime);
end;

procedure TGameClass.InitBeforeSceneFadeOut;
begin
  // Before Scene FadeOut ʱȭ
  InitFadeOut(Pointer(lytBeforeSceneMain), BeforeSceneFadeOutTime);
end;

function TGameClass.BeforeSceneFadeOut: Integer;
begin
  // Before Scene FadeOut
  // Result = 0:  , 1: 
  Result := RunFadeOut(Pointer(lytBeforeSceneMain), BeforeSceneFadeOutTime);
end;

procedure TGameClass.CupRollPress;
begin
  if intBeforeSceneIndex <> CUPROLL_PRESS then
  begin
    intBeforeSceneIndex := CUPROLL_PRESS;
  end;
end;

procedure TGameClass.GameStopPress;
begin
  case GetNowGameScene of
    BEFORE_SCENE:
      begin
        case intBeforeSceneIndex of
          GAMESTOP_PRESS, GAMESTOP_PRESS + 1:;
        else
          intBeforeSceneIndex := GAMESTOP_PRESS;
        end;
      end;
    RUN_SCENE:
      begin
        case intRunSceneIndex of
          GAMESTOP_PRESS, GAMESTOP_PRESS + 1:;
        else
          intRunSceneIndex := GAMESTOP_PRESS;
        end;
      end;
    AFTER_SCENE:
      begin
        case intAfterSceneIndex of
          GAMESTOP_PRESS, GAMESTOP_PRESS + 1:;
        else
          intAfterSceneIndex := GAMESTOP_PRESS;
        end;
      end;
  end;

end;

//==============================================================================
// RUN SCENE
//==============================================================================
procedure TGameClass.InitRunSceneIndex;
begin
  // Run Scene Index ʱȭ
  InitSceneIndex(intRunSceneIndex);
end;

procedure TGameClass.InitCupRollParameter;
begin
  //  ȸ īƮ
  intMainCupRollCount := 0;
  //  ȯ īƮ
  intSubCupRollCount := 0;
  // ӷ  ȸ 
  case intLevel of
    1: intCupRollLevel := 100 * 1;
    2: intCupRollLevel := 100 * 0.9;
    3: intCupRollLevel := 100 * 0.8;
    4: intCupRollLevel := 100 * 0.7;
    5: intCupRollLevel := 100 * 0.6;
    6: intCupRollLevel := 100 * 0.5;
    7: intCupRollLevel := 100 * 0.4;
    8: intCupRollLevel := 100 * 0.3;
    9: intCupRollLevel := 100 * 0.2;
    10: intCupRollLevel := 100 * 0.1;
  else
    intCupRollLevel := 100 * 0.01;
  end;
  // ȸ  ʿ 0
  intCupRollDelay := 0
end;

procedure TGameClass.InitCupRollDisplay;
var
  intLoop: Integer;
begin
  // ŵ  
  // Ball Visible False
  BallOnOff(False);

  GameCupList[0].Position.X := 67;
  GameCupList[0].Position.Y := 78;
  GameCupList[1].Position.X := 99;
  GameCupList[1].Position.Y := 78;
  GameCupList[2].Position.X := 131;
  GameCupList[2].Position.Y := 78;
  GameCupList[3].Position.X := 163;
  GameCupList[3].Position.Y := 78;

  // Cup Visible On
  for intLoop := 0 to 3 do
  begin
    GameCupList[intLoop].Visible := True;
  end;
end;

function TGameClass.RunCupRollAction(Index: Integer): Integer;
var
 X,Y: array[0..3] of Integer;

 imgCup01: TImage;
 imgCup11: TImage;
begin
  // ŵ
  // Result := 0  1: 
  Result := 0;

  X[0] := 5;  X[1] := 16;  X[2] := 27;  X[3] := 32;
  Y[0] := -5; Y[1] := -7;  Y[2] := -5;  Y[3] := 0;


  imgCup01 := GameCupList[Index + 0];
  imgCup11 := GameCupList[Index + 1];
  imgCup01.Position.X := (Index * 32 + X[intSubCupRollCount] + 67)       * 1;
  imgCup01.Position.Y := (Y[intSubCupRollCount] + 78)                    * 1;
  imgCup11.Position.X := ((Index + 1) * 32 - X[intSubCupRollCount] + 67) * 1;
  imgCup11.Position.Y := ((-Y[intSubCupRollCount]) + 78)                 * 1;

  Inc(intSubCupRollCount);
  if intSubCupRollCount > 3 then
  begin
    intSubCupRollCount := 0;
    GameCupList[Index + 1] := imgCup01;
    GameCupList[Index + 0] := imgCup11;
    Inc(intMainCupRollCount);
    if intMainCupRollCount > 49 then
    begin
      // δ .
      Result := 1;
    end;
  end;
end;

//==============================================================================
// AFTER SCENE
//==============================================================================
procedure TGameClass.InitAfterSceneIndex;
begin
  // After Scene Index ʱȭ
  InitSceneIndex(intAfterSceneIndex)
end;

function TGameClass.GetBallPosition: Integer;
begin
  // ġ  Ȯ
  Result := intBallPosition;
end;

function TGameClass.CheckBallPositionResult(Index: Integer): Integer;
begin
  // ġ ˻
  // Result := 0 Faile 1: OK
  Result := 0;

  case Index of
    CUPNUM1_PRESS: Index := 0;
    CUPNUM2_PRESS: Index := 1;
    CUPNUM3_PRESS: Index := 2;
    CUPNUM4_PRESS: Index := 3;
  else
    Exit;
  end;

  if intBallPosition = GameCupList[Index].Tag then
  begin
    Result := 1;
  end;
end;

procedure TGameClass.ResultBallDisplay;
var
  intX, intY: Integer;
begin
  // ġ ǥ(ŵ  ǥ)
  intY := 84;
  if GameCupList[0].Tag = intBallPosition then intX := 72;
  if GameCupList[1].Tag = intBallPosition then intX := 104;
  if GameCupList[2].Tag = intBallPosition then intX := 136;
  if GameCupList[3].Tag = intBallPosition then intX := 168;

  BallOnOff(True);
  imgBall.Position.X := intX;
  imgBall.Position.Y := intY;
end;

procedure TGameClass.ResultCupDisplay;
var
  intLoop: Integer;
begin
  // Cup Visible Off
  for intLoop := 0 to 3 do
  begin
    GameCupList[intLoop].Visible := False;
  end;

  GameCupList[0].Position.X := 67;
  GameCupList[0].Position.Y := 78;
  GameCupList[1].Position.X := 99;
  GameCupList[1].Position.Y := 78;
  GameCupList[2].Position.X := 131;
  GameCupList[2].Position.Y := 78;
  GameCupList[3].Position.X := 163;
  GameCupList[3].Position.Y := 78;

  for intLoop := 0 to 3 do
  begin
    if GameCupList[intLoop].Tag = intBallPosition then GameCupList[intLoop].Position.Y := 58;
  end;

  // Cup Visible On
  for intLoop := 0 to 3 do
  begin
    GameCupList[intLoop].Visible := True;
  end;
end;

procedure TGameClass.GameContinueOnOff(Visible: Boolean);
begin
  btnGameContinue.Visible := Visible;
end;

procedure TGameClass.CupNum1Press;
begin
  intAfterSceneIndex := CUPNUM1_PRESS;
end;

procedure TGameClass.CupNum2Press;
begin
  intAfterSceneIndex := CUPNUM2_PRESS;
end;

procedure TGameClass.CupNum3Press;
begin
  intAfterSceneIndex := CUPNUM3_PRESS;
end;

procedure TGameClass.CupNum4Press;
begin
  intAfterSceneIndex := CUPNUM4_PRESS;
end;

procedure TGameClass.GameContinuePress;
begin
  case intAfterSceneIndex of
    GAMECONTINUE_PRESS, GAMECONTINUE_PRESS + 1:;
  else
    intAfterSceneIndex := GAMECONTINUE_PRESS;
  end;
end;

//==============================================================================
// EXIT SCENE
//==============================================================================
procedure TGameClass.ChangeExitScene;
begin
  tbcMain.ActiveTab := tbtmExit;
  lytExitSceneMain.Opacity := 1;
end;

procedure TGameClass.InitExitSceneIndex;
begin
  // Exit Scene Index ʱȭ
  InitSceneIndex(intExitSceneIndex);
end;

procedure TGameClass.InitExitSceneFadeIn;
begin
  // Exit Scene FadeIn ʱȭ
  InitFadeIn(Pointer(lytExitSceneMain), ExitSceneFadeInTime);
end;

function TGameClass.ExitSceneFadeIn: Integer;
begin
  // Exit Scene FadeIn
  // Result = 0:  , 1: 
  Result := RunFadeIn(Pointer(lytExitSceneMain), ExitSceneFadeInTime);
end;

procedure TGameClass.InitExitSceneFadeOut;
begin
  // Exit Scene FadeOut ʱȭ
  InitFadeOut(Pointer(lytExitSceneMain), ExitSceneFadeOutTime);
end;

function TGameClass.ExitSceneFadeOut: Integer;
begin
  // Exit Scene FadeOut
  // Result = 0:  , 1: 
  Result := RunFadeOut(Pointer(lytExitSceneMain), ExitSceneFadeOutTime);
end;

procedure TGameClass.ExitOkPress;
begin
  case intExitSceneIndex of
    EXITOK_PRESS, EXITOK_PRESS + 1:;
    EXITCANCEL_PRESS, EXITCANCEL_PRESS + 1:;
  else
    intExitSceneIndex := EXITOK_PRESS;
  end;
end;

procedure TGameClass.ExitCancelPress;
begin
  case intExitSceneIndex of
    EXITOK_PRESS, EXITOK_PRESS + 1:;
    EXITCANCEL_PRESS, EXITCANCEL_PRESS + 1:;
  else
    intExitSceneIndex := EXITCANCEL_PRESS;
  end;
end;

//==============================================================================
// STOP SCENE
//==============================================================================
procedure TGameClass.ChangeStopScene;
begin
  tbcMain.ActiveTab := tbtmStop;
  lytStopSceneMain.Opacity := 1;
end;

procedure TGameClass.InitStopSceneIndex;
begin
  // Stop Scene Index ʱȭ
  InitSceneIndex(intStopSceneIndex);
end;

procedure TGameClass.InitStopSceneFadeIn;
begin
  // Stop Scene FadeIn ʱȭ
  InitFadeIn(Pointer(lytStopSceneMain), StopSceneFadeInTime);
end;

function TGameClass.StopSceneFadeIn: Integer;
begin
  // Stop Scene FadeIn
  // Result = 0:  , 1: 
  Result := RunFadeIn(Pointer(lytStopSceneMain), StopSceneFadeInTime);
end;

procedure TGameClass.InitStopSceneFadeOut;
begin
  // Stop Scene FadeOut ʱȭ
  InitFadeOut(Pointer(lytStopSceneMain), StopSceneFadeOutTime);
end;

function TGameClass.StopSceneFadeOut: Integer;
begin
  // Stop Scene FadeOut
  // Result = 0:  , 1: 
  Result := RunFadeOut(Pointer(lytStopSceneMain), StopSceneFadeOutTime);
end;

procedure TGameClass.StopOkPress;
begin
  case intStopSceneIndex of
    STOPOK_PRESS, STOPOK_PRESS + 1:;
    STOPCANCEL_PRESS, STOPCANCEL_PRESS + 1:;
  else
    intStopSceneIndex := STOPOK_PRESS;
  end;
end;

procedure TGameClass.StopCancelPress;
begin
  case intStopSceneIndex of
    STOPOK_PRESS, STOPOK_PRESS + 1:;
    STOPCANCEL_PRESS, STOPCANCEL_PRESS + 1:;
  else
    intStopSceneIndex := STOPCANCEL_PRESS;
  end;
end;


function TGameClass.WorkProcess: Integer;
begin
  // Result = 0:  , 1: 
  Result := 0;

  if Not bGameLoop then Exit;

  case nowGameScene of
    NONE_SCENE:
      begin
        // Intro Scene  
        SetGameScene(INTRO_SCENE);
        // Intro Scene Index ʱȭ
        InitIntroSceneIndex;
        // Intro ȭ ȯ
        ChangeIntroScene;
      end;
    INTRO_SCENE:
      begin
        case intIntroSceneIndex of
          NONE_READY:
            begin
              AllSceneScreenSetScale;
              InitIntroSceneFadeIn;
              Inc(intIntroSceneIndex);
            end;

          1:
            begin
              if IntroSceneFadeIn = 1 then
              begin
                InitIntroSceneFadeOut;
                Inc(intIntroSceneIndex);
              end;
            end;

          2:
            begin
              if IntroSceneFadeOut = 1 then
              begin
                // Main Scene  
                SetGameScene(MAIN_SCENE);
                // Main Scene Index ʱȭ
                InitMainSceneIndex;
                {
                SetGameScene(STOP_SCENE);
                InitStopSceneIndex;
                }
              end;
            end;
        end;
      end;

    MAIN_SCENE:
      begin
        case intMainSceneIndex of
          NONE_READY:
            begin
              // Main ȭ ȯ
              ChangeMainScene;
              intMainSceneIndex := STAND_BY;
            end;

          STAND_BY: ;

          GAMESTART_PRESS: // ӽ  
            begin
              // Main Scene ȭ FadeOut Ѵ.
              InitMainSceneFadeOut;
              intMainSceneIndex := GAMESTART_PRESS + 1
            end;

          GAMESTART_PRESS + 1:
            begin
              if MainSceneFadeOut = 1 then
              begin
                // Main Scene ȭ FadeOut Ǿ Before Scene ȭ Ѵ.
                SetGameScene(BEFORE_SCENE);
                // Before Scene Index ʱȭ
                InitBeforeSceneIndex;
                //  ʱȭ
                InitLevel;
              end;
            end;

          GAMEEND_PRESS: //   
            begin
              // MainScene ȭ FadeOut Ѵ.
              InitMainSceneFadeOut;
              intMainSceneIndex := GAMEEND_PRESS + 1;
            end;

          GAMEEND_PRESS + 1:
            begin
              if MainSceneFadeOut = 1 then
              begin
                // Main Scene ȭ FadeOut Ǿ Exit Scene ȭ Ѵ.
                SetGameScene(EXIT_SCENE);
                InitExitSceneIndex;
              end;
            end;
        end;
      end;

    BEFORE_SCENE:
      begin
        case intBeforeSceneIndex of
          NONE_READY:
            begin
              // ȭ鿡  ǥ
              LevelDisplay;

              // ġ 
              SetBallPosition;
              // ġ ǥ ( ܿ ǥ)
              BallDisplay;

              // ӻ ޼ 
              SetGameTopMessage(BEFORE_SCENE_TOP_MSG);

              CupRollButtonOnOff(True);
              GameContinueOnOff(False);
              GameStopButtonOnOff(True);
              CupButtonEnableOnOff(False);

              //    ӿ ʱȭ
              InitCup;
              //  ġ ǥ ( ǥ)
              InitCupDisplay;

              intBeforeSceneIndex := STAND_BY;
            end;

          STAND_BY: ChangeBeforeScene;

          CUPROLL_PRESS: //   
            begin
              SetGameScene(RUN_SCENE);
              InitRunSceneIndex;
            end;

          GAMESTOP_PRESS: //    
            begin
              // Before Scene ȭ FadeOut Ѵ.
              InitBeforeSceneFadeOut;
              intBeforeSceneIndex := GAMESTOP_PRESS + 1;
            end;

          GAMESTOP_PRESS + 1:
            begin
              if BeforeSceneFadeOut = 1 then
              begin
                // Before Scene ȭ FadeOut Ǿ Stop Scene ȭ Ѵ.
                SetGameScene(STOP_SCENE);
                InitStopSceneIndex;
              end;
            end;
        end;
      end;

    RUN_SCENE:
      begin
        case intRunSceneIndex of
          NONE_READY:
            begin
              //  ȸ  ʱȭ
              InitCupRollParameter;
              // ŵ  
              InitCupRollDisplay;
              // CupRoll, GameContinue  Off
              CupRollButtonOnOff(False);
              GameContinueOnOff(False);
              // Top ޼ 
              SetGameTopMessage(CUPROLL_MSG);

              intRunSceneIndex := CUP_ROLL;
            end;

          CUP_ROLL:
            begin
              ChangeBeforeScene;

              if TThread.GetTickCount > intCupRollDelay then
              begin
                //   Źȣ  
                if intSubCupRollCount = 0 then
                begin
                  IntSubCupRollIndex := Random(3);
                end;

                if RunCupRollAction(IntSubCupRollIndex) = 1 then
                begin
                  //  ȸ 
                  SetGameScene(AFTER_SCENE);
                  InitAfterSceneIndex;
                end
                else
                begin
                  //  ȸ,  缳
                  intCupRollDelay := TThread.GetTickCount + Round(intCupRollLevel);
                end;
              end;
            end;

          //==============================================================================
          // BeforeScene, RunScene, AfterScene ȭ  ϹǷ
          // BeforeScene ǵ FadeInOut ״ Ѵ.
          //==============================================================================
          GAMESTOP_PRESS: //    
            begin
              // Run Scene ȭ FadeOut Ѵ.
              InitBeforeSceneFadeOut;
              intRunSceneIndex := GAMESTOP_PRESS + 1;
            end;

          GAMESTOP_PRESS + 1:
            begin
              if BeforeSceneFadeOut = 1 then
              begin
                // Run Scene ȭ FadeOut Ǿ Stop Scene ȭ Ѵ.
                SetGameScene(STOP_SCENE);
                InitStopSceneIndex;
              end;
            end;
          //==============================================================================
        end;
      end;

    AFTER_SCENE:
      begin
        case intAfterSceneIndex of
          NONE_READY:
            begin
              //   ޼ 
              SetGameTopMessage(AFTER_SCENE_TOP_MSG);
              CupButtonEnableOnOff(True);
              intAfterSceneIndex := STAND_BY;
            end;

          STAND_BY: ChangeBeforeScene;


          CUPNUM1_PRESS,
          CUPNUM2_PRESS,
          CUPNUM3_PRESS,
          CUPNUM4_PRESS:
            begin
              if CheckBallPositionResult(intAfterSceneIndex) = 0 then
              begin
                // Ʋ
                SetGameTopMessage(BALLCHECK_FAIL_MSG);
              end
              else
              begin
                // ¾
                SetGameTopMessage(BALLCHECK_OK_MSG);
                //  
                IncLevel;
              end;

              CupButtonEnableOnOff(False);
              GameContinueOnOff(True);

              // ġ ǥ(ŵ  ǥ)
              ResultBallDisplay;

              ResultCupDisplay;

              intAfterSceneIndex := STAND_BY;
            end;

          GAMECONTINUE_PRESS: //  ϱ
            begin
              // Before Scene  
              SetGameScene(BEFORE_SCENE);
              // Before Scene Index ʱȭ
              InitBeforeSceneIndex;
              //  
            end;

          //==============================================================================
          // BeforeScene, RunScene, AfterScene ȭ  ϹǷ
          // BeforeScene ǵ FadeInOut ״ Ѵ.
          //==============================================================================
          GAMESTOP_PRESS:
            begin
              // Before Scene ȭ FadeOut Ѵ.
              InitBeforeSceneFadeOut;
              intAfterSceneIndex := GAMESTOP_PRESS + 1;
            end;

          GAMESTOP_PRESS + 1:
            begin
              if BeforeSceneFadeOut = 1 then
              begin
                // Before Scene ȭ FadeOut Ǿ Stop Scene ȭ Ѵ.
                SetGameScene(STOP_SCENE);
                InitStopSceneIndex;
              end;
            end;
          //==============================================================================

        end;


      end;

    EXIT_SCENE:
      begin
        case intExitSceneIndex of
          NONE_READY:
            begin
              // Exit Scene ȭ 
              ChangeExitScene;
              intExitSceneIndex := STAND_BY;
            end;

          STAND_BY:;

          EXITOK_PRESS:
            begin
              // α׷ 
              Result := 1;
            end;

          EXITCANCEL_PRESS:
            begin
              // Exit Scene ȭ FadeOut Ѵ.
              InitExitSceneFadeOut;
              intExitSceneIndex := EXITCANCEL_PRESS + 1;
            end;

          EXITCANCEL_PRESS + 1:
            begin
              if ExitSceneFadeOut = 1 then
              begin
                // Exit Scene ȭ FadeOut Ǿ,  Scene  Ѵ.
                SetGameScene(GetOldGameScene);
                // Scene Index  StandBy  Ѵ.
                SetGameSceneIndexToStandBy(GetNowGameScene);
              end;
            end;
        end;
      end;

    STOP_SCENE:
      begin
        case intStopSceneIndex of
          NONE_READY:
            begin
              // Stop Scene ȭ 
              ChangeStopScene;
              intStopSceneIndex := STAND_BY;
            end;

          STAND_BY: ;

          STOPOK_PRESS: //  OK 
            begin
              InitStopSceneFadeOut;
              intStopSceneIndex := STOPOK_PRESS + 1;
            end;

          STOPOK_PRESS + 1:
            begin
              if StopSceneFadeOut = 1 then
              begin
                // Stop Scene FadeOut Ǿ
                // Main Scene  Ѵ.
                SetGameScene(MAIN_SCENE);
                InitMainSceneIndex;
              end;
            end;

          STOPCANCEL_PRESS: //  Cancel 
            begin
              InitStopSceneFadeOut;
              intStopSceneIndex := STOPCANCEL_PRESS + 1;
            end;

          STOPCANCEL_PRESS + 1:
            begin
              if StopSceneFadeOut = 1 then
              begin
                // Stop Scene ȭ FadeOut Ǿ,  Scene  Ѵ.
                SetGameScene(GetOldGameScene);
                // Scene Index  StandBy  Ѵ.
                SetGameSceneIndexToStandBy(GetNowGameScene);
              end;
            end;
        end;
      end;
  end;

end;


end.
